package com.example.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.example.base.BaseController;
import com.example.base.ResponseBase;
import com.example.base.ResultVO;
import com.example.entity.LoginRequest;
import com.example.entity.MyWxMpUser;
import com.example.enums.ResultEnum;
import com.example.util.WxUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.Objects;

@Slf4j
@Controller
@RequestMapping("/wechat/app")
@Api(tags = "微信小程序api服务")
public class WechatAppController  extends BaseController {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());


    @Autowired
    private WxMaService wxMaService;



    /**
     * 登陆接口
     */
    @ApiOperation(value = "微信登录", notes = "微信登录")
    @PostMapping("/login")
    @ResponseBody
    public ResponseBase login(@RequestBody LoginRequest request) {
        String code = request.getCode();
        if (StringUtils.isBlank(code)) {
            return responseError(500,"请输入code错误!");
        }
        try {
            // 获取微信用户session
            WxMaJscode2SessionResult session = wxMaService.getUserService().getSessionInfo(request.getCode());
            if (null == session) {
                throw new RuntimeException("login handler error");
            }
            WxMaUserInfo wxUserInfo = JSON.parseObject(request.getRawData(), WxMaUserInfo.class);
            if (null == wxUserInfo) {
                throw new RuntimeException("wxUser not exist");
            }

            MyWxMpUser myWxMpUser = new MyWxMpUser();
            //TODO 可以增加自己的逻辑，关联业务相关数据
            myWxMpUser.setOpenId(session.getOpenid());
            myWxMpUser.setUnionId(session.getUnionid());
            myWxMpUser.setNickname(wxUserInfo.getNickName());
            myWxMpUser.setHeadImgUrl(wxUserInfo.getAvatarUrl());
            myWxMpUser.setSex(Integer.parseInt(wxUserInfo.getGender()));
            //后台新增用户成功
//            Map<String, String> tokenMap = customerInfoService.wxlogin(myWxMpUser);
            //登录系统获取本地
//            return R.success(tokenMap);
            return responseSuccess(new ResultVO());
        } catch (WxErrorException e) {
            this.logger.error(e.getMessage(), e);
            return responseError(500,"登录错误!");
        }
    }

//    @Autowired
//    private CustomerService customerService;

    /**
     * 登陆接口
     */
    @ApiOperation(value = "小程序绑定手机号", notes = "小程序绑定手机号")
    @PostMapping("/login/phone")
    @ResponseBody
    public ResponseBase loginphone(@RequestBody LoginRequest request) throws WxErrorException {
        // 获取微信用户session
        WxMaJscode2SessionResult session = wxMaService.getUserService().getSessionInfo(request.getCode());
        //解密手机号
        WxMaPhoneNumberInfo phoneNoInfo = wxMaService.getUserService().getPhoneNoInfo(session.getSessionKey(), request.getEncryptedData(), request.getIv());

        //todo:各业务逻辑处理
//        CustomerInfo customerInfo = new CustomerInfo();
//        CustomerInfoForm tokenUser = customerService.getCurrentMember();
//        customerInfo.setGuid(tokenUser.getGuid());
//        customerInfo.setPhone(phoneNoInfo.getPhoneNumber());
//        int i = customerInfoService.updateByPrimaryKeySelective(customerInfo);
//        if (i == 0) {
//            return resultVOError();
//        }
        //登录系统获取本地
        return responseSuccess(new ResultVO());
    }

    /**
     * 登陆接口
     */
    @ApiOperation(value = "小程序获取微信步数", notes = "小程序获取微信步数")
    @PostMapping("/steps")
    @ResponseBody
    public ResponseBase steps(@RequestBody LoginRequest request) throws WxErrorException {
        // 获取微信用户session
        WxMaJscode2SessionResult session = wxMaService.getUserService().getSessionInfo(request.getCode());
        String decrypt = null;
        //解密
        try {
            decrypt = WxUtils.decryptWeChatRunInfo(session.getSessionKey(), request.getEncryptedData(), request.getIv());
        } catch (Exception e) {
            e.printStackTrace();
        }

        if (decrypt == null) {
            log.info("解密失败");
        } else {
            log.info("解析成功");
        }
        JSONObject stepInfoListJson = JSON.parseObject(decrypt);
        //解析json，获取stepInfoList下面的步数
        JSONArray stepInfoList = JSON.parseArray(stepInfoListJson.getString("stepInfoList"));
        //获取今天的步数
        JSONObject today = Objects.requireNonNull(stepInfoList).getJSONObject(0);
        //登录系统获取本地
        return responseSuccess(new ResultVO(ResultEnum.SUCCESS,today));
    }


}
