package com.example.controller;

import com.alibaba.fastjson.JSONObject;
import com.example.base.BaseController;
import com.example.base.ResponseBase;
import com.example.base.ResultVO;
import com.example.base.Token;
import com.example.constant.Constants;
import com.example.constant.GlobleConstant;
import com.example.entity.User;
import com.example.enums.ResultEnum;
import com.example.service.UserService;
import com.example.util.RedisUtil;
import com.example.util.TokenUtil;
import com.example.vo.UserUpdateVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpOAuth2AccessToken;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;

@Slf4j
@Controller
@RequestMapping("/wechat/h5")
@Api(tags = "公众号网页登录")
public class WechatH5Controller extends BaseController {

    private static final String WX_USER = "wx_user_";

    /**
     * h5
     */
    @Autowired
    private WxMpService wxMpService;

    @Value("${jwt.tokenHeader}")
    private String tokenHeader;
    @Value("${jwt.tokenHead}")
    private String tokenHead;


    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private UserService userService;


    private static final String WECHAT_USER_INFO = "wechatUserInfo";


    @Value("${wx.mp.url}")//本地后台地址
    private String url;
    @Value("${wx.mp.before}")//前端首页地址
    private String reUrl;

    /**
     * 请求微信服务器认证授权
     *
     * @param returnUrl 微信认证授权时的state参数
     * @return
     */


    @ApiOperation(value = "微信认证")
    @GetMapping("/authorize")
    public void authorize(@RequestParam("returnUrl") String returnUrl, HttpServletResponse response) throws IOException {
        if (!StringUtils.isEmpty(returnUrl)) {
            reUrl = returnUrl;
        }
        // 用户授权完成后的重定向链接，441aca54.ngrok.io为使用ngrok代理后的公网域名，该域名需要替换为在微信公众号后台设置的域名，否则请求微信服务器不成功，一般都是采用将本地服务代理映射到一个可以访问的公网进行开发调试
        String redirectURL = wxMpService.oauth2buildAuthorizationUrl(url, WxConsts.OAuth2Scope.SNSAPI_USERINFO, URLEncoder.encode(returnUrl, "utf-8"));
        log.info("【微信网页授权】获取code,redirectURL={}", redirectURL);
//        redirectURL ="https://www.baidu.com/";
        response.sendRedirect(redirectURL);
    }

    @Autowired
    ApplicationContext applicationContext;

    /**
     * 认证授权成功后返回微信用户信息
     *
     * @param code      微信授权成功后重定向地址后的code参数
     * @param returnUrl 请求微信授权时携带的state参数
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "微信认证成功后回调")
    @GetMapping("/userInfo")
    public String userInfo(@RequestParam("code") String code,
                           @RequestParam("state") String returnUrl) throws Exception {
        log.info("【微信网页授权】code={}", code);
        log.info("【微信网页授权】state={}", returnUrl);
        WxMpOAuth2AccessToken wxMpOAuth2AccessToken;
        try {
            // 获取accessToken
            wxMpOAuth2AccessToken = wxMpService.oauth2getAccessToken(code);
        } catch (WxErrorException e) {
            log.error("【微信网页授权】{}", e);
            throw new Exception(e.getError().getErrorMsg());
        }
        // 根据accessToken获取openId
        String openId = wxMpOAuth2AccessToken.getOpenId();
        log.info("【微信网页授权】openId={}", openId);
        //applicationContext.publishEvent(new LoginOutEvent(this,wxMpOAuth2AccessToken.getAccessToken()));
        // 获取用户信息
        WxMpUser wxMpUser = wxMpService.oauth2getUserInfo(wxMpOAuth2AccessToken, null);
        log.info("【微信网页授权】wxMpUser={}", wxMpUser);
        // 刷新accessToken
        wxMpService.oauth2refreshAccessToken(wxMpOAuth2AccessToken.getRefreshToken());
        // 验证accessToken是否有效
        wxMpService.oauth2validateAccessToken(wxMpOAuth2AccessToken);

        redisUtil.set(WX_USER + openId, wxMpUser);

        User user = userService.getUserByOpenId(wxMpUser.getOpenId());
        if (user == null) {
            userService.addUser(wxMpUser);
        }

        Token token = TokenUtil.createJwtToken(Constants.LOGIN_TOKEN_KEY+user.getOpenId(), GlobleConstant.CURRENT_MEMBER);
        String accessToken = token.getAccessToken();
        System.err.println("生成的token:"+accessToken);
        redisUtil.set(accessToken, wxMpUser);
//        resultMap.put("accessToken", accessToken);

//        tokenService.createToken();




//        UmsMemberPlatform umsMemberPlatform= new UmsMemberPlatform();
//        String nickname = wxMpUser.getNickname();
//        wxMpUser.setNickname(EmojiUtil.filterEmoji(nickname));
//        BeanUtil.copyProperties(wxMpUser,umsMemberPlatform);
//        umsMemberPlatform.setOpenid(wxMpUser.getOpenId());
//        umsMemberPlatform.setUnionid(wxMpUser.getUnionId());
//        umsMemberPlatform.setHeadimgurl(wxMpUser.getHeadImgUrl());
//
//        String token = umsMemberPlatformService.addUmsMemberPlatformService(umsMemberPlatform);
        String trueUrl = null;
//        if (StringUtils.isEmpty(returnUrl)){
//            trueUrl= reUrl+"?token="+token+"&tokenHead="+tokenHead;
//        }else if (!returnUrl.contains("?")){
//            trueUrl=returnUrl+"?token="+token+"&tokenHead="+tokenHead;
//        }else {
//            trueUrl=returnUrl+"&token="+token+"&tokenHead="+tokenHead;
//        }
        return "redirect:" + returnUrl + "?token=" + accessToken;

    }

    @GetMapping("/user/get")
    @ApiOperation("获取微信用户信息")
    @ResponseBody
    public ResponseBase getUserInfo(HttpServletRequest request) {
        String openId = TokenUtil.parseJWT(request.getHeader("Authorization")).get("uid").toString().replace(Constants.LOGIN_TOKEN_KEY,"");
        User user = userService.getUserByOpenId(openId);
        return responseSuccess(new ResultVO(ResultEnum.SUCCESS, user));
    }

    @GetMapping("/code/{tel}")
    @ApiOperation("获取验证码")
    @ResponseBody
    public ResponseBase getCode(@PathVariable(value = "tel") String tel) {
        JSONObject json = new JSONObject();
        String code = (int) (Math.random() * 9000 + 1000)+"";
        json.put("code", code);
        redisUtil.set("phone_code_"+tel,code,300000);
        return responseSuccess(success(json));
    }

    @PostMapping("/user/update")
    @ApiOperation("修改用户手机号")
    @ResponseBody
    public ResponseBase updateUserPhone(@RequestBody UserUpdateVO updateVO,HttpServletRequest request){
        String openId = TokenUtil.parseJWT(request.getHeader("Authorization")).get("uid").toString().replace(Constants.LOGIN_TOKEN_KEY,"");
        String checkCode = (String) redisUtil.get("phone_code_"+updateVO.getPhone());
        if(StringUtils.isEmpty(checkCode)){
            return responseError(1,"验证码错误或已过期");
        }
        if(!checkCode.equals(updateVO.getCode())){
            return responseError(1,"验证码不匹配");
        }
        redisUtil.del("phone_code_"+updateVO.getPhone());
        userService.updateUser(openId,updateVO);
        return responseSuccess(success());
    }

}
